;example of fitting an edge-on galaxy using the markwardt package and
;        the code in this directory


gal=readfits('example_gal.fits') ;cropped and masked image of FGC 1285
weight=gal*0+1
weight(where(gal eq 0))=0 ;equal weighting on all the unmasked pixels

init_guess=[2.,100., 20.] ;numbers I made up
uncert=0
makex, gal,x,y  ;set up coordinate grid

parinfo=replicate({value:0.D, fixed:0}, 3)

fit=mpfit2dfun('one_disk', x, y, gal, uncert, init_guess, $
               weight=weight, perror=err, xtol=1.D-2, maxiter=30, $
               status=status, parinfo=parinfo, bestnorm=bestnorm)

;so here's the call to the magic markward code.  Most of the first bit
;   should be pretty clear.  The uncertainty variable is ignored
; because I used the weight keyword.  The perror keyword sticks the
; formal uncertainties of each parameter in the variable err.  This
; only works 1) if you use correct statistical weights and 2) you
; believe reduced chi^2 should equal 1.  For any lumpy nearby galaxy,
; you probably CANNOT assume chi^2/DoF=1.  The xtol keyword keeps the
; fitting from going to a huge number of significant figures.  

;this isn't how I actually fit edge-on galaxies because I'm interested
;     in getting the parameters for the faintest regions where it
; should be a nice mix of thin and thick flux.  Therefore, I use a
; weighting that puts a lot of weight where the galaxy is faint, but
; not too faint.  I just use the model to get an initiall weighting
; scheme, then loop over the mpfit2dfun call a couple times, useing a
; refined weigting each time.

print, 'final fit parameters= ', fit
fake_gal=one_disk(x,y,fit)

print, 'Displaying galaxy, fit, and residuals.'

!p.multi=[0,1,3]
imdisp, gal
imdisp, fake_gal
imdisp, (gal-fake_gal)*weight

print, ''
print, 'type .c to continue'
stop
;what the hell, let's see if the 2-disk thing works too
init_guess=[fit, fit, 0,0,0]
parinfo=replicate({value:0.D, fixed:0}, 9)

fit=mpfit2dfun('two_disk', x, y, gal, uncert, init_guess, $
               weight=weight, perror=err, xtol=1.D-2, maxiter=30, $
               status=status, parinfo=parinfo, bestnorm=bestnorm)

print, 'two disk fit params= ', fit
print, 'that sucked, try again with better initial guess'
print, ''
print, 'type .c to continue'
stop
init_guess=[fit(0:5),0,0,0]
fit=mpfit2dfun('two_disk', x, y, gal, uncert, init_guess, $
               weight=weight, perror=err, xtol=1.D-2, maxiter=30, $
               status=status, parinfo=parinfo, bestnorm=bestnorm)

print, 'So the 2-disk really needs a better weigting scheme if you want it to actually converge and be repeatable'
print, ''
print, 'type .c to continue'
stop
;try using inverse weigting to really fit the faint regions well.
nw=1./fake_gal
nw(where(weight eq 0))=0
nw(where(fake_gal lt .02))=0


;also hold the shift and rotation to zero
parinfo=replicate({value:0.D, fixed:0}, 9)
parinfo(6:8).fixed=1


fit=mpfit2dfun('two_disk', x, y, gal, uncert, [1.,30,14,.5,30,28,0,0,0], $
               weight=nw, perror=err, xtol=1.D-5, maxiter=30, $
               status=status, parinfo=parinfo, bestnorm=bestnorm)

;ack=two_disk(x,y,fit)

print, $
'Now the fit is starting to look like a bright thin disk and fainter thick disk'

fake_gal=two_disk(x,y,fit)
imdisp, gal
imdisp, fake_gal
imdisp, (gal-fake_gal)*weight


end

